using System.Text;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using ServerDatabase;
using ServerDatabase.SourceGenerator;
using bob = Luski.Shared.PublicServers.V1.Enums.PictureType;


namespace LuskiServer.Classes.TableDef;

public static class Channels
{
    public static TableColumn<ChannelRow, long> ID { get; } = new("id", true);
    public static TableColumn<ChannelRow, long> Parent { get; } = new("parent")
    {
        DefaultValue = 0 ,
        ForeignKey = new ForeignKey()
        {
            Table = Tables.Categories,
            Column = Categories.ID
        }
    };
    public static TableColumn<ChannelRow, ChannelType> Type { get; } = new("type");
    public static TableColumn<ChannelRow, DateTime> Epoch { get; } = new("epoch");
    public static TableColumn<ChannelRow, byte[]> Name { get; } = new("name") { DefaultValue = Encoding.UTF8.GetBytes("New Channel") };
    public static TableColumn<ChannelRow, byte[]> Description { get; } = new("description") { DefaultValue = Encoding.UTF8.GetBytes("New Channel") };
    public static TableColumn<ChannelRow, long> TitleEncryptionKey { get; } = new("title_encryption_key")
    {
        DefaultValue = 0,
        ForeignKey = new ForeignKey()
        {
            Table = Tables.Keys,
            Column = Keys.ID
        }
    };
    public static TableColumn<ChannelRow, long> DescriptionEncryptionKey { get; } = new("description_encryption_key")
    {
        DefaultValue = 0,
        ForeignKey = new ForeignKey()
        {
            Table = Tables.Keys,
            Column = Keys.ID
        }
    };
    public static TableColumn<ChannelRow, EncoderType> TitleEncoderType { get; } = new("title_encoder_type") { DefaultValue = EncoderType.UTF8 };
    public static TableColumn<ChannelRow, EncoderType> DescriptionEncoderType { get; } = new("description_encoder_type") { DefaultValue = EncoderType.UTF8 };
    public static TableColumn<ChannelRow, PictureType> PictureType { get; } = new("picture_type") {DefaultValue = bob.png };
    public static TableColumn<ChannelRow, byte[]> Picture { get; } = new("picture");
    public static TableColumn<ChannelRow, EncoderType[]> EncoderTypes { get; } = new("encoder_types") { DefaultValue = new [] 
    { 
        EncoderType.UTF8, EncoderType.UTF16, 
        EncoderType.UTF32, EncoderType.ASCII,
        EncoderType.Latin1, EncoderType.BigEndianUnicode 
    } };
}

[TableRow(typeof(Channels))]
public partial class ChannelRow;