using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class OfflineDataController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            List<string> bbb = new();
            if (Tables.OfflineData.TryReadRows(out var rows, OfflineData.StorageID.CreateParameter(SID)))
            {
                foreach (var blob in rows)
                {
                    bbb.Add(Convert.ToBase64String(blob.Data));
                }
            }
            
            Tables.OfflineData.DeleteRow(OfflineData.StorageID.CreateParameter(SID));
            //Tables.Storage.CreateCommand().WithFilter(Storage.ID, SID).WithValue(Storage.Data, Array.Empty<byte[]>()).Update();
            return this.ResponseToResult(new OfflineDataBlobSTC()
            {
                Data = bbb.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult GetInfo([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc; 
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            bool update = false;
            if (Tables.Sessions.Read(Sessions.StorageID, Sessions.ID.CreateParameter(SSID)) == 0)
            {
                SessionsRow[] rows = Tables.Sessions.ReadRows(uint.MaxValue, Sessions.ID.CreateParameter(SSID), Sessions.StorageID.CreateParameter(SID));
                if (rows.Length == 0)
                {
                    update = true;
                }
                Tables.Sessions.CreateCommand().WithFilter(Sessions.ID, SID).WithValue(Sessions.StorageID, SID).Update();
            }
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = Convert.ToBase64String(row.Password),
                ID = row.ID,
                Update = update
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult SetInfo([FromBody]StorageInfoCTS si, [FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            foreach (var h in HttpContext.Request.Headers)
            {
                foreach (var v in h.Value)
                {
                    Console.WriteLine(h.Key + ": " + v);
                }
            }
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Storage.CreateCommand().WithFilter(Storage.ID, SID).WithValue(Storage.Password, Convert.FromBase64String(si.Password)).Update();
            
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = si.Password,
                ID = row.ID,
                Update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult PostInfo([FromBody]StorageInfoCTS si, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            Console.WriteLine("POST");
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            LuskiFunctions.Snowflake sf = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch);
            Tables.Storage.Insert(
                Storage.ID.CreateParameter(sf.ID),
                Storage.Password.CreateParameter(Convert.FromBase64String(si.Password)),
                Storage.AccountID.CreateParameter(UID),
                Storage.OfflineKey.CreateParameter(0));
            Tables.Sessions.CreateCommand().WithFilter(Sessions.ID, sf.ID).WithValue(Sessions.ID, SSID).Update();
            
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = si.Password,
                ID = sf.ID,
                Update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}