using System.Net.Mime;
using System.Security.Cryptography;
using System.Text;
using Asp.Versioning;
using JacobTechEncryption;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketAccountController : Controller
{
    /// <summary>
    /// Make a get request to this endpoint to login to your account on the server.
    /// </summary>
    /// <param name="UsernameRaw">Encrypted email for account.</param>
    /// <param name="PasRaw">Hashed Password for account.</param>
    /// <param name="KeyRaw">Plain text RSA key for server to respond with.</param>
    /// <returns></returns>
    [HttpGet]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(LoginSTC), StatusCodes.Status201Created)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "username")]string UsernameRaw, [FromHeader(Name = "password")]string PasRaw, [FromHeader(Name = "key")]string KeyRaw)
    {
        try
        {
            byte[] PasBytes, Username;
            try
            {
                try
                {
                    
                    Username = Encryption.RSA.Decrypt(Convert.FromBase64String(UsernameRaw), LuskiFunctions.Encryption.Keys.PrivateKey);
                    if (!Tables.Users.TryRead(Users.Salt, out byte[]? salt, Users.Username.CreateParameter(Username)))
                        return this.ShowError(ErrorCode.Forbidden, "Make sure our login info is correct");
                    PasBytes = LuskiFunctions.Encryption.RemotePasswordEncrypt(PasRaw, salt);

                    try
                    {
                        byte[] g = Encryption.RSA.Encrypt("Test data to send to client", KeyRaw, EncoderType.UTF8);
                    }
                    catch
                    {
                        return this.ShowError(ErrorCode.InvalidHeader, "The public keys you gave cant be used to encrypt data");
                    }
                }
                catch
                {
                    return this.ShowError(ErrorCode.InvalidHeader, "Make sure your login is encrypted with the server provided public key");
                }
            }
            catch (Exception ex)
            {
                if (ex.Message.Contains("not found")) return this.ShowError(ErrorCode.MissingHeader, "Missing login infermation");
                return this.ShowError(ex);
            }

            if (!Tables.Users.TryRead(Users.ID, out long ID, Users.Username.CreateParameter(Username),
                    Users.Password.CreateParameter(PasBytes)))
                return this.ShowError(ErrorCode.Forbidden, "Make sure our login info is correct");
            int num = new Random().Next(1000, 1000000000);
            int num2 = new Random().Next(1000, 1000000000);
            byte[] Timestamp = Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString());
            byte[] Number = Encoding.UTF8.GetBytes(num.ToString());
            byte[] Number2 = Encoding.UTF8.GetBytes(num2.ToString());
            string Token = $"{Convert.ToBase64String(Encoding.UTF8.GetBytes(ID.ToString()))}.{Convert.ToBase64String(Timestamp)}.{Convert.ToBase64String(Number)}.{Convert.ToBase64String(Number2)}";
            
            LuskiFunctions.Snowflake id = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch);
            long kid = id.ID;
            Tables.Users.CreateCommand().WithFilter(Users.ID, ID).WithValue(Users.Status, UserStatus.Invisible).Update();
            Tables.Keys.Insert(
                Keys.ID.CreateParameter(kid),
                Keys.EncryptionType.CreateParameter(EncryptionType.RSA),
                Keys.KeyData.CreateParameter(Encoding.UTF8.GetBytes(KeyRaw)));
            Thread t = new(o => RegToken((string?)o, kid));
            t.Start(Token);
            return StatusCode(StatusCodes.Status202Accepted, new LoginSTC()
            {
                Token = Token
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    /// <summary>
    /// Make a get request to this endpoint to login to your account on the server.
    /// </summary>
    /// <param name="UsernameRaw">Encrypted email for account.</param>
    /// <param name="PasRaw">Hashed Password for account.</param>
    /// <param name="KeyRaw">Plain text RSA key for server to respond with.</param>
    /// <returns></returns>
    [HttpGet]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(LoginSTC), StatusCodes.Status201Created)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/AccessToken")]
    public IActionResult GetT([FromHeader(Name = "token")]string TokenRaw, [FromHeader(Name = "key")]string KeyRaw)
    {
        try
        {
            string tok;
            try
            {
                    
                tok = Encryption.RSA.Decrypt(Convert.FromBase64String(TokenRaw),
                    LuskiFunctions.Encryption.Keys.PrivateKey, EncoderType.UTF16);

                try
                {
                    byte[] g = Encryption.RSA.Encrypt("Test data to send to client", KeyRaw, EncoderType.UTF8);
                }
                catch
                {
                    return this.ShowError(ErrorCode.InvalidHeader, "The public keys you gave cant be used to encrypt data");
                }
            }
            catch
            {
                return this.ShowError(ErrorCode.InvalidHeader, "Make sure your login is encrypted with the server provided public key");
            }

            if (!Tables.SessionTokens.TryRead(SessionTokens.AccountID, out long ID, SessionTokens.Token.CreateParameter(tok)))
                return this.ShowError(ErrorCode.Forbidden, "Make sure our login info is correct");
            int num = new Random().Next(1000, 1000000000);
            int num2 = new Random().Next(1000, 1000000000);
            byte[] Timestamp = Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString());
            byte[] Number = Encoding.UTF8.GetBytes(num.ToString());
            byte[] Number2 = Encoding.UTF8.GetBytes(num2.ToString());
            string Token = $"{Convert.ToBase64String(Encoding.UTF8.GetBytes(ID.ToString()))}.{Convert.ToBase64String(Timestamp)}.{Convert.ToBase64String(Number)}.{Convert.ToBase64String(Number2)}";
            Tables.SessionTokens.DeleteRow(SessionTokens.Token.CreateParameter(tok));
            LuskiFunctions.Snowflake id = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch);
            long kid = id.ID;
            Tables.Users.CreateCommand().WithFilter(Users.ID, ID).WithValue(Users.Status, UserStatus.Invisible).Update();
            Console.WriteLine("C");
            Tables.Keys.Insert(
                Keys.ID.CreateParameter(kid),
                Keys.EncryptionType.CreateParameter(EncryptionType.RSA),
                Keys.KeyData.CreateParameter(Encoding.UTF8.GetBytes(KeyRaw)));
            Thread t = new(o => RegToken((string?)o, kid));
            t.Start(Token);
            return StatusCode(StatusCodes.Status202Accepted, new LoginSTC()
            {
                Token = Token
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    /// <summary>
    /// Make a post request to this endpoint for registering an account on the server.
    /// </summary>
    /// <param name="UsernameRaw">Encrypted email for account.</param>
    /// <param name="PasRaw">Hashed Password for account.</param>
    /// <param name="DisplayName">Plain text Username.</param>
    /// <param name="KeyRaw">Plain text RSA key for server to respond with.</param>
    /// <param name="Body">Raw Picture data for profile.</param>
    /// <returns></returns>
    [HttpPost]
    [DisableRequestSizeLimit]
    //[Consumes(MediaTypeNames.Image.Jpeg, MediaTypeNames.Image.Gif, MediaTypeNames.Image.Tiff, "image/png")]
    //[Consumes(MediaTypeNames.Application.Octet)]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(LoginSTC), StatusCodes.Status201Created)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public async Task<IActionResult> Post([FromHeader(Name = "username")]string? UsernameRaw, [FromHeader(Name = "password")]string PasRaw, [FromHeader(Name = "displayname")]string? DisplayName, [FromHeader(Name = "key")]string? KeyRaw)
    {
        try
        {
            PictureType pfp;
            byte[] PasBytes, Username;
            byte[] salt = new byte[100], Body;
            using (MemoryStream reader = new())
            {
                await Request.Body.CopyToAsync(reader);
                Body = reader.ToArray();
            }
            try
            {
                using (RandomNumberGenerator provider = RandomNumberGenerator.Create())
                {
                    provider.GetBytes(salt);
                }
                try
                {
                    PasBytes = LuskiFunctions.Encryption.RemotePasswordEncrypt(PasRaw, salt);
                    Username = Encryption.RSA.Decrypt(Convert.FromBase64String(UsernameRaw), LuskiFunctions.Encryption.Keys.PrivateKey);

                    try
                    {
                        byte[] g = Encryption.RSA.Encrypt("Test data to send to client", KeyRaw, EncoderType.UTF8);
                    }
                    catch
                    {
                        return this.ShowError(ErrorCode.InvalidHeader, "The public keys you gave cant be used to encrypt data");
                    }
                }
                catch
                {
                    return this.ShowError(ErrorCode.InvalidHeader, "Make sure your login is encrypted with the server provided public key");
                }
            }
            catch (Exception ex)
            {
                if (ex.Message.Contains("not found")) return this.ShowError(ErrorCode.MissingHeader, "Missing login infermation");
                return this.ShowError(ex);
            }

            if (CheckUsername(Username))
            {
                int num = new Random().Next(1000, 1000000000);
                int num2 = new Random().Next(1000, 1000000000);
                LuskiFunctions.Snowflake id = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch);
                bool owner = false;
                if (Tables.Server.Read(Server.Owner, Server.ID.CreateParameter(0)) == -1)
                {
                    id = new(0);
                    Tables.Server.CreateCommand().WithValue(Server.Owner, id.ID).Update();
                    if (!Tables.Keys.TryRead(Keys.ID, out _, Keys.ID.CreateParameter(0)))
                    {
                        Tables.Keys.Insert(
                            Keys.ID.CreateParameter(0),
                            Keys.EncryptionType.CreateParameter(EncryptionType.None),
                            Keys.KeyData.CreateParameter(Array.Empty<byte>()));
                    }
                    if (!Tables.Categories.TryRead(Categories.ID, out _, Categories.ID.CreateParameter(0)))
                    {
                        Tables.Categories.Insert(
                            Categories.ID.CreateParameter(0),
                            Categories.Name.CreateParameter(Encoding.UTF8.GetBytes("server")),
                            Categories.Description.CreateParameter(
                                Encoding.UTF8.GetBytes("The default category for the server. Everybody will see this category."))
                        );
                        Tables.Channels.Insert(
                            Channels.ID.CreateParameter(0),
                            Channels.Type.CreateParameter(ChannelType.TextAndVoice),
                            Channels.Description.CreateParameter(Encoding.UTF8.GetBytes("Default chat for you to use in your new server")),
                            Channels.Name.CreateParameter(Encoding.UTF8.GetBytes("Default Channel")),
                            Channels.Epoch.CreateParameter(DateTime.UtcNow.Date),
                            Channels.TitleEncryptionKey.CreateParameter(0),
                            Channels.Parent.CreateParameter(0),
                            Channels.DescriptionEncryptionKey.CreateParameter(0),
                            Channels.TitleEncoderType.CreateParameter(EncoderType.UTF8),
                            Channels.DescriptionEncoderType.CreateParameter(EncoderType.UTF8),
                            Channels.PictureType.CreateParameter(PictureType.none),
                            Channels.Picture.CreateParameter(Array.Empty<byte>()),
                            Channels.EncoderTypes.CreateParameter(new [] { EncoderType.UTF8 }));
                        Tables.ChannelKeys.Insert(ChannelKeys.Channel.CreateParameter(0), ChannelKeys.Key.CreateParameter(0));
                    }
                    owner = true;
                }
                byte[] ID = Encoding.UTF8.GetBytes(id.ID.ToString());
                byte[] Timestamp = Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString());
                byte[] Number = Encoding.UTF8.GetBytes(num.ToString());
                byte[] Number2 = Encoding.UTF8.GetBytes(num2.ToString());
                string Token = $"{Convert.ToBase64String(ID)}.{Convert.ToBase64String(Timestamp)}.{Convert.ToBase64String(Number)}.{Convert.ToBase64String(Number2)}";

                pfp = GetProfilePictureType(Encoding.UTF8.GetString(Body).ToUpper());
                long uid = id.ID;
                long kid = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID;
                long pid = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID;
                Tables.Profiles.Insert(
                    Profiles.DisplayName.CreateParameter(DisplayName!.Replace("'", "\'")),
                    Profiles.ID.CreateParameter(pid),
                    Profiles.PictureType.CreateParameter(pfp),
                    Profiles.Picture.CreateParameter(Body),
                    Profiles.CanUpdateName.CreateParameter(true));
                Tables.Users.Insert(
                    Users.ID.CreateParameter(uid),
                    Users.ServerProfile.CreateParameter(pid),
                    Users.SelectedChannel.CreateParameter(0), //TODO set to default channel
                    Users.Status.CreateParameter(UserStatus.Online),
                    Users.Username.CreateParameter(Username),
                    Users.Password.CreateParameter(PasBytes),
                    Users.Salt.CreateParameter(salt));
                Tables.RoleMembers.Insert(RoleMembers.Role.CreateParameter(0), RoleMembers.User.CreateParameter(uid));
                Tables.ProfileControllers.Insert(
                    ProfileControllers.Profile.CreateParameter(pid),
                    ProfileControllers.User.CreateParameter(uid));
                Tables.CategoryProfiles.Insert(
                    CategoryProfiles.Profile.CreateParameter(pid),
                    CategoryProfiles.Category.CreateParameter(0));
                Tables.Keys.Insert(
                    Keys.ID.CreateParameter(kid),
                    Keys.EncryptionType.CreateParameter(EncryptionType.RSA),
                    Keys.KeyData.CreateParameter(Encoding.UTF8.GetBytes(KeyRaw)));
                Thread t = new(o => RegToken((string?)o, kid));
                t.Start(Token);
                return StatusCode(201, new LoginSTC()
                {
                    Token = Token
                });
            }
            return this.ShowError(ErrorCode.Forbidden, "That email is already being used");
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
    
    private static PictureType GetProfilePictureType(string headerCode)
    {
        if (headerCode.StartsWith("FFD8FFE0"))
        {
            return PictureType.jpeg;
        }
        else if (headerCode.StartsWith("49492A"))
        {
            return PictureType.tif;
        }
        else if (headerCode.StartsWith("424D"))
        {
            return PictureType.bmp;
        }
        else if (headerCode.StartsWith("GIF"))
        {
            return PictureType.gif;
        }
        else if (headerCode.Remove(0, 1).StartsWith("PNG"))
        {
            return PictureType.png;
        }
        else
        {
            return PictureType.png;
        }
    }
    
    /// <summary>
    /// Cheaks to see Username is free
    /// </summary>
    /// <param name="Username"></param>
    /// <returns>
    /// <para>true = Good</para>
    /// <para>false = Username Taken</para>
    /// </returns>
    private static bool CheckUsername(byte[] Username)
    {
        if (Tables.Users.TryRead(Users.Username, out _, Users.Username.CreateParameter(Username))) return false;
        return true;
    }

    private static void RegToken(string? token, long kid)
    {
        try
        {
            if (token == null) return;
            long id = long.Parse(Encoding.UTF8.GetString(Convert.FromBase64String(token.Split('.')[0])));
            long l = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID;
            Tables.Sessions.Insert(
                Sessions.ID.CreateParameter(l),
                Sessions.User.CreateParameter(id),
                Sessions.LoginToken.CreateParameter(token),
                Sessions.SessionKey.CreateParameter(kid),
                Sessions.WSSTCP.CreateParameter(string.Empty),
                Sessions.Token.CreateParameter(string.Empty),
                Sessions.StorageID.CreateParameter(0));
            Thread.Sleep(30000);
            if (Tables.Sessions.TryRead(Sessions.LoginToken, out string? tok, Sessions.ID.CreateParameter(l)) && tok == token)
                Tables.Sessions.DeleteRow(Sessions.ID.CreateParameter(l));
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
        }
    }
}