using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using Luski.Shared.PublicServers.V1.Shared;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketCategoryController : ControllerBase
{
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]CategoryPostCTS ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, ChanReq.Parent, out ServerPermission opt, ServerPermission.CreateCategories)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = LuskiFunctions.Config.ServerEpoch;

            epoch = DateTime.UtcNow.Date;
            LuskiFunctions.Snowflake chanSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
            Tables.Categories.Insert(
                Categories.ID.CreateParameter(chanSnowflake.ID),
                Categories.Name.CreateParameter(Convert.FromBase64String(ChanReq.Name)),
                Categories.Description.CreateParameter(Convert.FromBase64String(ChanReq.Description)),
                Categories.TitleEncryptionKey.CreateParameter(ChanReq.TitleEncryptionKey),
                Categories.DescriptionEncryptionKey.CreateParameter(ChanReq.DescriptionEncryptionKey),
                Categories.TitleEncoderType.CreateParameter(ChanReq.TitleEncoderType),
                Categories.DescriptionEncoderType.CreateParameter(ChanReq.DescriptionEncoderType));
            Tables.CategoryParents.Insert(CategoryParents.Parent.CreateParameter(ChanReq.Parent), CategoryParents.ID.CreateParameter(chanSnowflake.ID));
            
            if (ChanReq.RoleOverrides.Length > 0)
            {
                foreach (RoleOverrideCTS over in ChanReq.RoleOverrides)
                {
                    Tables.CategoryRoleOverrides.Insert(
                        CategoryRoleOverrides.Category.CreateParameter(chanSnowflake.ID),
                        CategoryRoleOverrides.RoleOverride.CreateParameter(over.RoleID),
                        CategoryRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        CategoryRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            }
            if (ChanReq.UserOverrides.Length > 0)
            {
                foreach (UserOverrideCTS over in ChanReq.UserOverrides)
                {
                    Tables.CategoryUserOverrides.Insert(
                        CategoryUserOverrides.Category.CreateParameter(chanSnowflake.ID),
                        CategoryUserOverrides.UserOverride.CreateParameter(over.UserID),
                        CategoryUserOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        CategoryUserOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            } 
            
            CategoryRow chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(chanSnowflake.ID));
            long[] ic = Array.Empty<long>();
            long[] c = Array.Empty<long>();
            if (!opt.HasPermission(ServerPermission.ViewCategories))
            {
                ic = Tables.CategoryParents.ReadColumn(CategoryParents.ID, CategoryParents.Parent.CreateParameter(chanSnowflake.ID));
            }
            if (!opt.HasPermission(ServerPermission.ViewChannels))
            {
                c = LuskiFunctions.GetCategoryChannels(chanSnowflake.ID, ID);
            }
            List<RoleOverrideSTC> RO = new();
            List<UserOverrideSTC> UO = new();
            foreach (var over in Tables.CategoryRoleOverrides.ReadRows(CategoryRoleOverrides.Category.CreateParameter(chanSnowflake.ID)))
            {
                RO.Add(new()
                {
                    RoleID = over.RoleOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            foreach (var over in Tables.CategoryUserOverrides.ReadRows(CategoryUserOverrides.Category.CreateParameter(chanSnowflake.ID)))
            {
                UO.Add(new()
                {
                    UserID = over.UserOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }

            long Parent = -1;
            if (Tables.CategoryParents.TryRead(CategoryParents.Parent, out long s,
                    CategoryParents.ID.CreateParameter(chan.ID)))
            {
                Parent = s;
            }
            
            return this.ResponseToResult(new CategorySTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = chanSnowflake.ID,
                Channels = c,
                InnerCategories = ic,
                Parent = Parent,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                UserOverrides = UO.ToArray(),
                RoleOverrides = RO.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Upload/Matrix")]
    public IActionResult PostMatrix([FromHeader(Name = "channel")]CategoryPostCTS ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, ChanReq.Parent, out ServerPermission opt, ServerPermission.CreateCategories)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = LuskiFunctions.Config.ServerEpoch;

            epoch = DateTime.UtcNow.Date;
            LuskiFunctions.Snowflake chanSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
            Tables.Categories.Insert(
                Categories.ID.CreateParameter(chanSnowflake.ID),
                Categories.Name.CreateParameter(Convert.FromBase64String(ChanReq.Name)),
                Categories.Description.CreateParameter(Convert.FromBase64String(ChanReq.Description)),
                Categories.TitleEncryptionKey.CreateParameter(ChanReq.TitleEncryptionKey),
                Categories.DescriptionEncryptionKey.CreateParameter(ChanReq.DescriptionEncryptionKey),
                Categories.TitleEncoderType.CreateParameter(ChanReq.TitleEncoderType),
                Categories.DescriptionEncoderType.CreateParameter(ChanReq.DescriptionEncoderType));
            Tables.CategoryParents.Insert(CategoryParents.Parent.CreateParameter(ChanReq.Parent), CategoryParents.ID.CreateParameter(chanSnowflake.ID));
            
            if (ChanReq.RoleOverrides.Length > 0)
            {
                foreach (RoleOverrideCTS over in ChanReq.RoleOverrides)
                {
                    Tables.CategoryRoleOverrides.Insert(
                        CategoryRoleOverrides.Category.CreateParameter(chanSnowflake.ID),
                        CategoryRoleOverrides.RoleOverride.CreateParameter(over.RoleID),
                        CategoryRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        CategoryRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            }
            if (ChanReq.UserOverrides.Length > 0)
            {
                foreach (UserOverrideCTS over in ChanReq.UserOverrides)
                {
                    Tables.CategoryUserOverrides.Insert(
                        CategoryUserOverrides.Category.CreateParameter(chanSnowflake.ID),
                        CategoryUserOverrides.UserOverride.CreateParameter(over.UserID),
                        CategoryUserOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        CategoryUserOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            } 
            
            CategoryRow chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(chanSnowflake.ID));
            long[] ic = Array.Empty<long>();
            long[] c = Array.Empty<long>();
            if (!opt.HasPermission(ServerPermission.ViewCategories))
            {
                ic = Tables.CategoryParents.ReadColumn(CategoryParents.ID, CategoryParents.Parent.CreateParameter(chanSnowflake.ID));
            }
            if (!opt.HasPermission(ServerPermission.ViewChannels))
            {
                c = LuskiFunctions.GetCategoryChannels(chanSnowflake.ID, ID);
            }
            List<RoleOverrideSTC> RO = new();
            List<UserOverrideSTC> UO = new();
            foreach (var over in Tables.CategoryRoleOverrides.ReadRows(CategoryRoleOverrides.Category.CreateParameter(chanSnowflake.ID)))
            {
                RO.Add(new()
                {
                    RoleID = over.RoleOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            foreach (var over in Tables.CategoryUserOverrides.ReadRows(CategoryUserOverrides.Category.CreateParameter(chanSnowflake.ID)))
            {
                UO.Add(new()
                {
                    UserID = over.UserOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }

            long Parent = -1;
            if (Tables.CategoryParents.TryRead(CategoryParents.Parent, out long s,
                    CategoryParents.ID.CreateParameter(chan.ID)))
            {
                Parent = s;
            }
            
            return this.ResponseToResult(new CategorySTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = chanSnowflake.ID,
                Channels = c,
                InnerCategories = ic,
                Parent = Parent,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                UserOverrides = UO.ToArray(),
                RoleOverrides = RO.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, ServerPermission.DeleteChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Channels.DeleteRow(Channels.ID.CreateParameter(Channel));
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Patch([FromHeader(Name = "id")]long Channel)  //TODO Add Channel edit latter
    {
        return null!;
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, out ServerPermission opt, ServerPermission.ViewCategories | ServerPermission.ViewChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            CategoryRow chan = Tables.Categories.ReadRow(Categories.ID.CreateParameter(Channel));
            List<long> ic = new();
            long[] c = Array.Empty<long>();
            if (!opt.HasPermission(ServerPermission.ViewCategories))
            {
                ic = Tables.CategoryParents.ReadColumn(CategoryParents.ID, CategoryParents.Parent.CreateParameter(Channel)).ToList();
            }
            if (!opt.HasPermission(ServerPermission.ViewChannels))
            {
                c = LuskiFunctions.GetCategoryChannels(Channel, ID);
            }

            foreach (long cc in ic)
            {
                if (!LuskiFunctions.HasAccessToChannel(ID, cc)) ic.Remove(cc);
            }
            List<RoleOverrideSTC> RO = new();
            List<UserOverrideSTC> UO = new();
            foreach (var over in Tables.CategoryRoleOverrides.ReadRows(CategoryRoleOverrides.Category.CreateParameter(chan.ID)))
            {
                RO.Add(new()
                {
                    RoleID = over.RoleOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            foreach (var over in Tables.CategoryUserOverrides.ReadRows(CategoryUserOverrides.Category.CreateParameter(chan.ID)))
            {
                UO.Add(new()
                {
                    UserID = over.UserOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            long Parent = -1;
            if (Tables.CategoryParents.TryRead(CategoryParents.Parent, out long s,
                    CategoryParents.ID.CreateParameter(chan.ID)))
            {
                Parent = s;
            }
            return this.ResponseToResult(new CategorySTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = Channel,
                Channels = c,
                InnerCategories = ic.ToArray(),
                Parent = Parent,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                UserOverrides = UO.ToArray(),
                RoleOverrides = RO.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}