using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketChannelController : ControllerBase
{
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]ChannelPostCTS ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, ChanReq.Parent, ServerPermission.CreateChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = LuskiFunctions.Config.ServerEpoch;
            //List<long> RoleOverides = new(), UserOverides = new();
            

            epoch = DateTime.UtcNow.Date;
            LuskiFunctions.Snowflake chanSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
            Tables.Channels.Insert(
                Channels.ID.CreateParameter(chanSnowflake.ID),
                Channels.Parent.CreateParameter(ChanReq.Parent),
                Channels.Type.CreateParameter(ChanReq.Type),
                Channels.Epoch.CreateParameter(epoch.Date),
                Channels.Name.CreateParameter(Convert.FromBase64String(ChanReq.Name)),
                Channels.Description.CreateParameter(Convert.FromBase64String(ChanReq.Description)),
                Channels.TitleEncryptionKey.CreateParameter(ChanReq.TitleEncryptionKey),
                Channels.DescriptionEncryptionKey.CreateParameter(ChanReq.DescriptionEncryptionKey),
                Channels.TitleEncoderType.CreateParameter(ChanReq.TitleEncoderType),
                Channels.DescriptionEncoderType.CreateParameter(ChanReq.DescriptionEncoderType),
                Channels.PictureType.CreateParameter(PictureType.none),
                Channels.Picture.CreateParameter(Array.Empty<byte>()),
                Channels.EncoderTypes.CreateParameter(ChanReq.EncoderTypes));
            foreach (long ek in ChanReq.EncryptionKeys)
            {
                Tables.ChannelKeys.Insert(ChannelKeys.Channel.CreateParameter(chanSnowflake.ID), ChannelKeys.Key.CreateParameter(ek));
            }
            
            if (ChanReq.RoleOverrides.Length > 0)
            {
                foreach (RoleOverrideCTS over in ChanReq.RoleOverrides)
                {
                    Tables.ChannelRoleOverrides.Insert(
                        ChannelRoleOverrides.Channel.CreateParameter(chanSnowflake.ID),
                        ChannelRoleOverrides.RoleOverride.CreateParameter(over.RoleID),
                        ChannelRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        ChannelRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            }
            if (ChanReq.UserOverrides.Length > 0)
            {
                foreach (UserOverrideCTS over in ChanReq.UserOverrides)
                {
                    Tables.ChannelUserOverrides.Insert(
                        ChannelUserOverrides.Channel.CreateParameter(chanSnowflake.ID),
                        ChannelUserOverrides.UserOverride.CreateParameter(over.UserID),
                        ChannelUserOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        ChannelUserOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                }
            }

            
            
            
            ChannelRow chan = Tables.Channels.ReadRow(Channels.ID.CreateParameter(chanSnowflake.ID));
            List<RoleOverrideSTC> RO = new();
            List<UserOverrideSTC> UO = new();
            foreach (var over in Tables.ChannelRoleOverrides.ReadRows(ChannelRoleOverrides.Channel.CreateParameter(chanSnowflake.ID)))
            {
                RO.Add(new()
                {
                    RoleID = over.RoleOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            foreach (var over in Tables.ChannelUserOverrides.ReadRows(ChannelUserOverrides.Channel.CreateParameter(chanSnowflake.ID)))
            {
                UO.Add(new()
                {
                    UserID = over.UserOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            
            return this.ResponseToResult(new ChannelSTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = chanSnowflake.ID,
                Type = chan.Type,
                Parent = chan.Parent,
                Epoch = chan.Epoch,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                EncryptionKeys = Tables.ChannelKeys.ReadColumn(ChannelKeys.Key, ChannelKeys.Channel.CreateParameter(chanSnowflake.ID)),
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                EncoderTypes = chan.EncoderTypes,
                UserOverrides = UO.ToArray(),
                RoleOverrides = RO.ToArray(),
                PictureType = chan.PictureType
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, ServerPermission.DeleteChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Channels.DeleteRow(Channels.ID.CreateParameter(Channel));
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Patch([FromHeader(Name = "id")]long Channel)  //TODO Add Channel edit latter
    {
        return null;
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/GetPicture")]
    public IActionResult GetPicture([FromHeader(Name = "id")]long chan)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, chan, ServerPermission.ViewThis)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            if (!Tables.Channels.TryRead(Channels.Picture, out byte[] image, Channels.ID.CreateParameter(chan))) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            return Tables.Channels.Read(Channels.PictureType, Channels.ID.CreateParameter(chan)) switch
            {
                PictureType.png => File(image, "image/png"),
                PictureType.jpeg => File(image, "image/jpeg"),
                PictureType.bmp => File(image, "image/bmp"),
                PictureType.gif => File(image, "image/gif"),
                PictureType.ico => File(image, "image/vnd.microsoft.icon"),
                PictureType.svg => File(image, "image/svg+xml"),
                PictureType.tif => File(image, "image/tiff"),
                PictureType.webp => File(image, "image/webp"),
                _ => File(image, "image/png"),
            };
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, Channel, ServerPermission.ViewChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            ChannelRow chan = Tables.Channels.ReadRow(Channels.ID.CreateParameter(Channel));
            List<RoleOverrideSTC> RO = new();
            List<UserOverrideSTC> UO = new();
            foreach (var over in Tables.ChannelRoleOverrides.ReadRows(ChannelRoleOverrides.Channel.CreateParameter(chan.ID)))
            {
                RO.Add(new()
                {
                    RoleID = over.RoleOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            foreach (var over in Tables.ChannelUserOverrides.ReadRows(ChannelUserOverrides.Channel.CreateParameter(chan.ID)))
            {
                UO.Add(new()
                {
                    UserID = over.UserOverride,
                    BadPermissions = over.BadPermissions,
                    GoodPermissions = over.GoodPermissions
                });
            }
            return this.ResponseToResult(new ChannelSTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = Channel,
                Type = chan.Type,
                Parent = chan.Parent,
                Epoch = chan.Epoch,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                EncryptionKeys = Tables.ChannelKeys.ReadColumn(ChannelKeys.Key, ChannelKeys.Channel.CreateParameter(Channel)),
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                EncoderTypes = chan.EncoderTypes,
                UserOverrides = UO.ToArray(),
                RoleOverrides = RO.ToArray(),
                PictureType = chan.PictureType
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}