using System.Net.Mime;
using System.Text;
using Asp.Versioning;
using JacobTechEncryption;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketFileController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromQuery(Name = "id")]long id)
    {
        try
        {
            if (!Tables.Files.TryRead(Files.Data, out byte[] raw, Files.ID.CreateParameter(id))) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            if (Tables.Files.Read(Files.Public, Files.ID.CreateParameter(id)))
            {
                FileRow r = Tables.Files.ReadRow(Files.ID.CreateParameter(id));
                if (r.EncryptionKey == 0)
                {
                    return File(raw, MediaTypeNames.Application.Octet, Encryption.Generic.Encoders[(int)r.NameEncoderType].GetString(r.Name));
                }
                else return File(raw, MediaTypeNames.Application.Octet);
            }
            else
            {
                if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
                return File(raw, MediaTypeNames.Application.Octet);
            }
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")] long File_id)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Files.TryRead(Files.Owner, out long raw, Files.ID.CreateParameter(File_id)) || raw != ID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Files.CreateCommand()
                .WithFilter(Files.ID, File_id)
                .WithValue(Files.Data, Array.Empty<byte>())
                .WithValue(Files.NameEncoderType, EncoderType.UTF8)
                .WithValue(Files.NameEncryptionKey, 0)
                .WithValue(Files.Name, Encoding.UTF8.GetBytes("Deleted File"))
                .WithValue(Files.Size, 0)
                .WithValue(Files.EncryptionKey, 0).Update();
            
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public async Task<IActionResult> Post([FromHeader(Name = "name_encoder")] EncoderType nameEncoderNum, [FromHeader(Name = "name_encryption")]long nameEncryptionRaw, [FromHeader(Name = "encoder")] EncoderType EncoderNum, [FromHeader(Name = "encryption")]long EncryptionRaw, [FromHeader(Name = "name")]string name)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            byte[] data;
            using (MemoryStream reader = new())
            {
                await Request.Body.CopyToAsync(reader);
                data = reader.ToArray();
            }
            byte[] hash = Encryption.Hashing.SHA256(data);
            if (Tables.Files.TryReadRow(out FileRow ido, Files.Hash.CreateParameter(hash)))
                return this.ResponseToResult(new ServerFileInfoSTC()
                {
                    ID = ido.ID,
                    Name = Convert.ToBase64String(ido.Name),
                    Encoder = ido.EncoderType,
                    NameEncoder = ido.NameEncoderType,
                    Key = ido.EncryptionKey,
                    NameKey = ido.NameEncryptionKey,
                    Size = ido.Size
                });
            long ido2 = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID;
            Tables.Files.Insert(
                Files.ID.CreateParameter(ido2),
                Files.Size.CreateParameter(data.Length),
                Files.Name.CreateParameter(Convert.FromBase64String(name)),
                Files.NameEncoderType.CreateParameter(nameEncoderNum),
                Files.NameEncryptionKey.CreateParameter(nameEncryptionRaw),
                Files.Data.CreateParameter(data),
                Files.EncoderType.CreateParameter(EncoderNum),
                Files.EncryptionKey.CreateParameter(EncryptionRaw),
                Files.Hash.CreateParameter(hash),
                Files.Owner.CreateParameter(ID),
                Files.Public.CreateParameter(false));
            return this.ResponseToResult(new ServerFileInfoSTC()
            {
                ID = ido2,
                Name = name,
                Encoder = EncoderNum,
                NameEncoder = nameEncoderNum,
                Key = EncryptionRaw,
                NameKey = nameEncryptionRaw,
                Size = data.Length
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    //TODO Add File edit latter
}