using System.Net.Mime;
using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using Luski.Shared.PublicServers.V1.Shared;
using LuskiServer.Classes;
using LuskiServer.Classes.ServerComs;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums.ServerComs;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketMessageController : ControllerBase
{
    private IActionResult GetMSG(long MSG_id)
    {
        MessageRow msgraw = Tables.Messages.ReadRow(Messages.ID.CreateParameter(MSG_id));
        MessageSTC msg = new()
        {
            AuthorID = msgraw.AuthorID,
            Context = Convert.ToBase64String(msgraw.Context),
            ID = msgraw.ID,
            ChannelID = msgraw.ChannelID,
            ProfileID = msgraw.ProfileID
        };
        List<ServerFileInfoSTC> list = new();
        if (Tables.MessageFiles.CreateCommand().WithFilter(MessageFiles.MessageID, msgraw.ID)
            .WithCrossTableCheck(MessageFiles.FileID, Tables.Files, Files.ID).TryReadRows(out FileRow[]? files))
        {
            foreach (FileRow b in files)
            {
                ServerFileInfoSTC file = new()
                {
                    ID = b.ID,
                    Name = Convert.ToBase64String(b.Name),
                    Size = b.Size,
                    NameEncoder = b.NameEncoderType,
                    NameKey = b.NameEncryptionKey,
                    Encoder = b.EncoderType,
                    Key = b.EncryptionKey,
                };
                list.Add(file);
            }
        }
        msg.Files = list.ToArray();

        return this.ResponseToResult(msg);
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "msg_id")]long MSG_id)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Messages.TryRead(Messages.ChannelID, out long Channel_Id, Messages.ID.CreateParameter(MSG_id)) ||
                !LuskiFunctions.HasAccessToChannel(ID, Channel_Id, ServerPermission.ReadMessageHistory)) this.ResponseCodeToResult(ErrorCode.Forbidden);
            MessageRow msgraw = Tables.Messages.ReadRow(Messages.ID.CreateParameter(MSG_id));
            MessageSTC msg = new()
            {
                AuthorID = msgraw.AuthorID,
                Context = Convert.ToBase64String(msgraw.Context),
                ID = MSG_id,
                ChannelID = Channel_Id,
                ProfileID = msgraw.ProfileID
            };
            List<ServerFileInfoSTC> list = new();
            if (Tables.MessageFiles.CreateCommand().WithFilter(MessageFiles.MessageID, msgraw.ID)
                .WithCrossTableCheck(MessageFiles.FileID, Tables.Files, Files.ID).TryReadRows(out FileRow[]? files))
            {
                foreach (FileRow b in files)
                {
                    ServerFileInfoSTC file = new()
                    {
                        ID = b.ID,
                        Name = Convert.ToBase64String(b.Name),
                        Size = b.Size,
                        NameEncoder = b.NameEncoderType,
                        NameKey = b.NameEncryptionKey,
                        Encoder = b.EncoderType,
                        Key = b.EncryptionKey,
                    };
                    list.Add(file);
                }
            }
            msg.Files = list.ToArray();
            return this.ResponseToResult(msg);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [DisableRequestSizeLimit]
    [Consumes(typeof(MessageCTS), MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]MessageCTS data)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, data.ChannelID, ServerPermission.SendMessages)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            LuskiFunctions.Snowflake Id = LuskiFunctions.Snowflake.GenerateSnowflake();
            ChannelType type = Tables.Channels.Read(Channels.Type, Channels.ID.CreateParameter(data.ChannelID));
            bool Fake = data.Profile.HasValue;
            long profile = Tables.Users.Read(Users.ServerProfile, Users.ID.CreateParameter(ID));
            if (Fake) //TODO check to make sure the profile can be used by them
            {
                profile = data.Profile!.Value;
            }
            MessageEvent MessageEvent = new()
            {
                AuthorID = ID,
                ProfileID = profile,
                ChannelID = data.ChannelID,
                Base64Context = data.Base64Context,
                ID = Id.ID,
                EncoderType = data.Encoding,
                EncryptionKey = data.EncryptionKey,
                Files = Array.Empty<ServerFileInfoSTC>(),
                Timestamp = LuskiFunctions.Info.GetTimestampFromEpoch(Tables.Channels.Read(Channels.Epoch, Channels.ID.CreateParameter(data.ChannelID)))
            };
           
            Tables.Messages.Insert(
                Messages.ChannelID.CreateParameter(data.ChannelID),
                Messages.ID.CreateParameter(Id.ID),
                Messages.Timestamp.CreateParameter(MessageEvent.Timestamp),
                Messages.AuthorID.CreateParameter(ID),
                Messages.ProfileID.CreateParameter(profile),
                Messages.Context.CreateParameter(Convert.FromBase64String(data.Base64Context)),
                Messages.EncoderType.CreateParameter(data.Encoding),
                Messages.EncryptionKey.CreateParameter(data.EncryptionKey),
                Messages.ReplyTo.CreateParameter(Id.ID));
            if (data.Files.Length > 0)
            {
                List<ServerFileInfoSTC> SFI = new();
                foreach (long file in data.Files)
                {
                    if (Tables.Files.TryReadRow(out FileRow ido, Files.ID.CreateParameter(file)))
                    {
                        Tables.MessageFiles.Insert(MessageFiles.MessageID.CreateParameter(Id.ID), MessageFiles.FileID.CreateParameter(file));
                        SFI.Add(new ServerFileInfoSTC()
                        {
                            ID = ido.ID,
                            Name = Convert.ToBase64String(ido.Name),
                            Encoder = ido.EncoderType,
                            NameEncoder = ido.NameEncoderType,
                            Key = ido.EncryptionKey,
                            NameKey = ido.NameEncryptionKey,
                            Size = ido.Size
                        });
                    }
                        
                }

                MessageEvent.Files = SFI.ToArray();
            }
            WSS.SendData(SendType.ID_Group, new ServerEvent()
            {
                Type = DataType.MessageCreate,
                Data = MessageEvent
            }, LuskiFunctions.GetChannelBroadcastMembers(data.ChannelID));
            Response.StatusCode = 201;
            return GetMSG(Id.ID);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    //TODO Add Message edit latter
}