using Asp.Versioning;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketUserController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Person)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (Person != ID && !Tables.Users.TryRead(Users.ID, out _, Users.ID.CreateParameter(Person))) return this.ShowError(ErrorCode.Forbidden, "User does not exist");
            UserRow UR = Tables.Users.ReadRow(Users.ID.CreateParameter(Person));
            if (Person == ID) return this.ResponseToResult(new SocketUserSTC()
            {
                ID = Person,
                RoleIds = Tables.RoleMembers.ReadColumn(RoleMembers.Role, RoleMembers.User.CreateParameter(Person)),
                Status = UR.Status,
                SelectedChannel = UR.SelectedChannel,
                ServerProfile = UR.ServerProfile
            });
            return this.ResponseToResult(new SocketUserSTC()
            {
                ID = Person,
                RoleIds = Tables.RoleMembers.ReadColumn(RoleMembers.Role, RoleMembers.User.CreateParameter(Person)),
                Status = UR.Status,
                ServerProfile = UR.ServerProfile
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}