using System.IO.Compression;
using System.Text.Json;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Converters.Matrix.Types;

namespace LuskiServer.Converters.Matrix;

public static class Matrix
{
    /// <summary>
    /// 
    /// </summary>
    /// <param name="Zip">File Stream</param>
    /// <param name="Channel">Luski Channel ID</param>
    /// <param name="defaultuser">ID of user doing the conversion</param>
    /// <param name="UserMap">Map of matrix name to luski account</param>
    /// <param name="ProfilesIdList">Map of matrix name to Luski Profile</param>
    /// <param name="Profiles">Map of matrix names to new Luski Profiles</param>
    /// <param name="IgnoredUsers">List of ignored matrix names</param>
    /// <param name="Encoder">The Encoding to use for Luski Messages</param>
    /// <param name="Encryption">The Encryption key to use for Luski Messages</param>
    /// <param name="remove">Removes channel if same id is already created</param>
    /// <exception cref="Exception"></exception>
    public static void ZipJsonChatExport(Stream Zip, long Channel, long defaultuser, Dictionary<string, long> UserMap,
        Dictionary<string, int> ProfilesIdList,
        List<Tuple<string, byte[], PictureType>> Profiles,
        List<string> IgnoredUsers, EncoderType Encoder, long Encryption, bool remove = true)
    {
        Dictionary<int, long> IdConverter = new();
        Dictionary<string, long> cps = new();
        if (Tables.Channels.TryRead(Channels.ID, out _, Channels.ID.CreateParameter(Channel)))
        {
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            //throw new Exception("Channel Exist");
        }
        
        using ZipArchive archive = new ZipArchive(Zip, ZipArchiveMode.Read, false);
        string internalfolder = archive.Entries[0].FullName;
        ZipArchiveEntry json = archive.GetEntry(internalfolder + "export.json")!;
        MessageExport export = JsonSerializer.Deserialize(json.Open(), MessageExportContext.Default.MessageExport)!;

        void checkFakeUser(string id)
        {
            if (!IdConverter.ContainsKey(ProfilesIdList[id]))
            {
                long id2 = LuskiFunctions.Snowflake.GenerateSnowflake().ID;
                        
                Tables.Profiles.Insert(
                    Classes.TableDef.Profiles.ID.CreateParameter(id2),
                    Classes.TableDef.Profiles.PictureType.CreateParameter(Profiles[ProfilesIdList[id]].Item3),
                    Classes.TableDef.Profiles.Picture.CreateParameter(Profiles[ProfilesIdList[id]].Item2),
                    Classes.TableDef.Profiles.DisplayName.CreateParameter(Profiles[ProfilesIdList[id]].Item1));
                IdConverter.Add(ProfilesIdList[id], id2);
                Tables.ChannelProfiles.Insert(ChannelProfiles.Channel.CreateParameter(Channel), ChannelProfiles.Profile.CreateParameter(id2));
                foreach (KeyValuePair<string, int> v in ProfilesIdList.Where(s => s.Value == ProfilesIdList[id]))
                {
                    cps.Add(v.Key, id2);
                }
            }
        }

        DateTime epoch = DateTime.UnixEpoch.AddMilliseconds(export.Messages[0].Origin).Date;

        try
        {
            Tables.Channels.CreateCommand()
                .WithFilter(Channels.ID, Channel)
                .WithValue(Channels.Parent, 0)
                .WithValue(Channels.Type, ChannelType.TextAndVoice)
                .WithValue(Channels.Description, export.RoomTopic.ToDB(Encoder, Encryption))
                .WithValue(Channels.Name, export.RoomName.ToDB(Encoder, Encryption))
                .WithValue(Channels.Epoch, epoch)
                .WithValue(Channels.TitleEncryptionKey, Encryption)
                .WithValue(Channels.DescriptionEncryptionKey, Encryption)
                .WithValue(Channels.TitleEncoderType, Encoder)
                .WithValue(Channels.DescriptionEncoderType, Encoder)
                .WithValue(Channels.EncoderTypes, new[] { Encoder }).Update();
            

            Dictionary<string, long> Events = new();
            int inow = 0;
            int imax = export.Messages.Length;
            foreach (Message msg in export.Messages)
            {
                inow++;
                Console.WriteLine(inow + " of " + imax);
                if (msg.Type != "m.room.message")
                    continue;
                if (!UserMap.ContainsKey(msg.Sender) && !ProfilesIdList.ContainsKey(msg.Sender))
                {
                    if (IgnoredUsers.Contains(msg.Sender))
                        continue;
                    if (remove)
                    {
                        Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
                    }

                    throw new Exception($"User not in list: {msg.Sender}");
                }


                DateTime cur = DateTime.UnixEpoch.AddMilliseconds(msg.Origin);
                if (msg.Context.Type is null)
                    continue;
                LuskiFunctions.Snowflake sf = LuskiFunctions.Snowflake.GenerateSnowflake(epoch, cur);
                long reply = sf.ID;
                string body = msg.Context.Body;
                
                Events.Add(msg.Event, sf.ID);
                cur = cur.ToLocalTime();
                long id;
                long profile = 0;
                if (ProfilesIdList.ContainsKey(msg.Sender))
                {
                    checkFakeUser(msg.Sender);
                    profile = IdConverter[ProfilesIdList[msg.Sender]];
                    id = defaultuser;
                }
                else
                {
                    id = UserMap[msg.Sender];
                    profile = Tables.Users.Read(Users.ServerProfile, Users.ID.CreateParameter(id));
                }
                
                if (msg.Context.Relations is not null)
                {
                    if (Events.TryGetValue(msg.Context.Relations.Reply.ID, out long greply)) reply = greply;
                    //Console.WriteLine("Body: {0}", msg.Context.Body);
                    //Console.WriteLine("Fancy Body: {0}", msg.Context.FormattedBody);
                    // string[] ars = msg.Context.FormattedBody.Split('\n');
                    if (msg.Context.FormattedBody is null) body = msg.Context.Body;
                    else body = msg.Context.FormattedBody.Split("</mx-reply>")[^1]
                        .Replace("<br/>", "\n")
                        .Replace("<br>", "\n")
                        .Replace("&#x27;", "'")
                        .Replace("&#39;", "'")
                        .Replace("\\", "\\\\")
                        .Replace("[", "\\[");
                    if (body.Contains("could build a better dropdown than Discord"))
                    {
                        Console.WriteLine("break");
                    }
                    
                    if (msg.Context.Mentions is not null && msg.Context.Mentions.IDs is not null && msg.Context.Mentions.IDs.Length > 0)
                    {
                        foreach (var i in msg.Context.Mentions.IDs)
                        {
                            string test = "[";
                            if (UserMap.ContainsKey(i))
                            {
                                test += $"@{UserMap[i]}]";
                            }
                            else
                            {
                                checkFakeUser(i);
                                test += $"%{cps[i]}>";
                            }
                            body = body.Replace($"<a href=\"https://matrix.to/#/{i}\"",test);
                            int index = -1;
                            while (true)
                            {
                                index = body.IndexOf(test, index + 1, StringComparison.Ordinal);
                                if (index == -1) break;
                                int index2 = body.IndexOf("</a>", index + 1, StringComparison.Ordinal);
                                body = body.Remove(index + test.Length, index2 + 4 - test.Length - index);
                            }
                        }
                    }

                    body = body.Replace("<p>", "").Replace("</p>", "\n");
                    if (body.EndsWith("\n")) body = body.Remove(body.Length - 1, 1);
                    //body = msg.Context.FormattedBody;
                    Console.WriteLine(body);
                }
                else
                {
                    if (msg.Context.Mentions is not null && msg.Context.Mentions.IDs is not null && msg.Context.Mentions.IDs.Length > 0)
                    {
                        body = msg.Context.FormattedBody!;
                        foreach (var i in msg.Context.Mentions.IDs)
                        {
                            string test = "[";
                            if (UserMap.ContainsKey(i))
                            {
                                test += $"@{UserMap[i]}>";
                            }
                            else
                            {
                                checkFakeUser(i);
                                test += $"%{cps[i]}]";
                            }
                            body = body!.Replace($"<a href=\"https://matrix.to/#/{i}\"",test);
                            int index = -1;
                            while (true)
                            {
                                index = body.IndexOf(test, index + 1, StringComparison.Ordinal);
                                if (index == -1) break;
                                int index2 = body.IndexOf("</a>", index + 1, StringComparison.Ordinal);
                                body = body.Remove(index + test.Length, index2 + 4 - test.Length - index);
                            }
                        }
                        body = body!.Replace("<p>", "").Replace("</p>", "\n");
                        if (body.EndsWith("\n")) body = body.Remove(body.Length - 1, 1);
                    }
                }
                
                switch (msg.Context.Type)
                {
                    default:
                        Tables.Messages.Insert(
                            Messages.ID.CreateParameter(sf.ID),
                            Messages.Context.CreateParameter(body.ToDB(Encoder, Encryption)),
                            Messages.ChannelID.CreateParameter(Channel),
                            Messages.EncoderType.CreateParameter(Encoder),
                            Messages.EncryptionKey.CreateParameter(Encryption),
                            Messages.AuthorID.CreateParameter(id),
                            Messages.ReplyTo.CreateParameter(reply),
                            Messages.ProfileID.CreateParameter(profile));
                        break;
                    case "m.image" or "m.video" or "m.audio" or "m.file":
                        int h = cur.Hour;
                        string pm = "AM";
                        if (h > 11)
                        {
                            h = h - 12;
                            if (h == 0) h = 12;
                            pm = "PM";
                        }

                        string hh = h.ToString();
                        string mm = cur.Minute.ToString();
                        string s = cur.Second.ToString();
                        if (s.Length == 1) s = "0" + s;
                        if (mm.Length == 1) mm = "0" + mm;
                        if (hh.Length == 1) hh = "0" + hh;
                        string[] spl = msg.Context.Body.Split('.');
                        string fil =
                            $"{spl[0]}-{cur.Month}-{cur.Day}-{cur.Year} at {h}-{mm}-{s} {pm}.{spl[spl.Length - 1]}";
                        string loc = msg.Context.Type.Split('.')[1] + "s";
                        if (loc == "audios") loc = "audio";
                        if (fil.EndsWith(".1")) fil = fil.Remove(fil.Length - 2, 2);
                        ZipArchiveEntry? f;
                        if ((f = archive.GetEntry(internalfolder + loc + "/" + fil)) is not null)
                        {
                            byte[] d;
                            using(var memoryStream = new MemoryStream())
                            {
                                f.Open().CopyTo(memoryStream);
                                d = memoryStream.ToArray();
                            }

                            Tables.Files.Insert(
                                Files.ID.CreateParameter(sf.ID),
                                Files.Name.CreateParameter(msg.Context.Body.ToDB(Encoder, Encryption)),
                                Files.Data.CreateParameter(d),
                                Files.EncoderType.CreateParameter(Encoder),
                                Files.EncryptionKey.CreateParameter(Encryption),
                                Files.Hash.CreateParameter(JacobTechEncryption.Encryption.Hashing.SHA256(d)),
                                Files.Owner.CreateParameter(id),
                                Files.Public.CreateParameter(true),
                                Files.Size.CreateParameter(d.LongLength),
                                Files.NameEncoderType.CreateParameter(Encoder),
                                Files.NameEncryptionKey.CreateParameter(Encryption));
                            Tables.FileChannels.Insert(FileChannels.ChannelID.CreateParameter(Channel), FileChannels.FileID.CreateParameter(sf.ID));
                            Tables.Messages.Insert(
                                Messages.ID.CreateParameter(sf.ID),
                                Messages.Context.CreateParameter(Array.Empty<byte>()),
                                Messages.ChannelID.CreateParameter(Encryption),
                                Messages.EncoderType.CreateParameter(Encoder),
                                Messages.EncryptionKey.CreateParameter(Encryption),
                                Messages.AuthorID.CreateParameter(id),
                                Messages.ReplyTo.CreateParameter(reply));
                            Tables.MessageFiles.Insert(MessageFiles.FileID.CreateParameter(sf.ID), MessageFiles.MessageID.CreateParameter(sf.ID));
                        }
                        break;
                }
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
            if (remove)
            {
                Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            }
            throw;
        }
    }
}