using System.Reflection;
using System.Text;
using Asp.Versioning.ApiExplorer;
using BenchmarkDotNet.Running;
using JacobTechEncryption;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer;
using LuskiServer.Classes;
using LuskiServer.Classes.ServerComs;
using LuskiServer.Classes.TableDef;
using LuskiServer.Converters.Matrix;
using LuskiServer.Converters.Matrix.Types;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ServerDatabase;
using ServerDatabase.Utils;
using Swashbuckle.AspNetCore.SwaggerGen;

[assembly: Microsoft.AspNetCore.Mvc.ApiController]

LuskiFunctions.Config = LuskiServer.Classes.LuskiFunctions.GetSettings("/etc/luskiserver/app.json", AppConfigContext.Default.AppConfig, true);
List<string> tttt = args.ToList();
if (tttt.Contains("--urls"))
{
    tttt[tttt.IndexOf("--urls") + 1] = string.Join(";", LuskiFunctions.Config.Addresses);
}
else
{
    tttt.Add("--urls");
    tttt.Add(string.Join(";", LuskiFunctions.Config.Addresses));
}

args = tttt.ToArray();
Console.WriteLine($"Server starting with starting epoch of {LuskiFunctions.Config.ServerEpoch.ToLocalTime()} and current time of {DateTime.UtcNow.ToLocalTime()}");
LuskiFunctions.Database = new Database(LuskiFunctions.Config.Address, 
    LuskiFunctions.Config.Database, 
    LuskiFunctions.Config.Username, 
    LuskiFunctions.Config.Password,
    LuskiFunctions.Config.CustomeName);




Dictionary<string, List<Message>> fff = new();


LuskiFunctions.Database.TestDatabase();

foreach (PropertyInfo prop in typeof(Tables).GetProperties())
{
    IBetterTable table = (IBetterTable)prop.GetValue(null)!;
    table.DatabaseHandler = LuskiFunctions.Database;
    Type t = Type.GetType($"LuskiServer.Classes.TableDef.{prop.Name}")!;
    foreach(PropertyInfo prop2 in t.GetProperties())
    {
        table.AddColumn((ITableColumn)prop2.GetValue(null)!, prop2.Name);
    }
    LuskiFunctions.Database.Tables.Add(table);
}

LuskiFunctions.Database.RegisterTables();

//Tables.ProfileColors.CreateCommand().WithFilter(ProfileColors.Profile, 0).WithValue(ProfileColors.ColorType, ColorType.LeftToRightGradient).WithValue(ProfileColors.Color, Convert.FromHexString("E54D24FFDDB249FF")).Update();

if (!Tables.Server.TryRead(Server.ID, out _, Server.ID.CreateParameter(0)))
{
    Tables.Server.Insert();
}

if (!Tables.Profiles.TryRead(Profiles.ID, out _, Profiles.ID.CreateParameter(0)))
{
    Tables.Profiles.Insert(
        Profiles.ID.CreateParameter(0),
        Profiles.DisplayName.CreateParameter("Server"),
        Profiles.PictureType.CreateParameter(PictureType.none),
        Profiles.Picture.CreateParameter(Array.Empty<byte>()));
}

if (!LuskiFunctions.Database.VersionsTable.TryRead(LuskiFunctions.Database.VersionsTable.ID, out _,
        LuskiFunctions.Database.VersionsTable.ID.CreateParameter(0)))
{
    LuskiFunctions.Database.VersionsTable.Insert(LuskiFunctions.Database.VersionsTable.ID.CreateParameter(0));
}


/*
Dictionary<string, long> AMap = new()
{
    {"@jacobtech:matrix.org", 0},
    //{"@c4tz:matrix.org", 2},
    //{"@themagicalcats:matrix.org", 2},
    //{"@captainleader:matrix.org", 1},
    //{"@kaizenash:matrix.org", 1},
    //{"@quadro:matrix.org", 4},
    //{"@bubs_cooper:matrix.org", 3}
};

Dictionary<string, int> BMap = new()
{
    {"@rqndomnezz:matrix.org",0},
    {"@ardillybar:matrix.org", 1},
    {"@themagicalcats:matrix.org", 2},
    {"@c4tz:matrix.org", 2},
    {"@jobotnik:matrix.org", 3},
    {"@captainleader:matrix.org", 4},
    {"@kaizenash:matrix.org", 4},
    {"@bubs_cooper:matrix.org", 5},
    {"@quadro:matrix.org", 6},
    {"@tcll:matrix.org", 7},
    {"@eeetile:matrix.org", 8},
    {"@etile:matrix.org", 8},
    {"@tranquillity_:midov.pl", 9},
    {"@troler:asra.gr", 10},
    {"@xnq:matrix.org", 11},
    {"@jacobtech-bot:matrix.org", 12}
};

List<Tuple<string, byte[], PictureType, byte[]>> BMapL = new()
{
    new("Rqndomnezz", File.ReadAllBytes("/home/jacob/Downloads/2023-01-20_23-29.png"), PictureType.png, new byte[] { 161, 196, 248, 255 }),
    new("ardillybar", File.ReadAllBytes("/home/jacob/Downloads/a.jpeg"), PictureType.jpeg, new byte[] { 255, 168, 188, 255 }),
    new("Cats", File.ReadAllBytes("/home/jacob/Downloads/c.png"), PictureType.png, new byte[] { 114, 213, 174, 255 }),
    new("jobotnik", File.ReadAllBytes("/home/jacob/Downloads/j.png"), PictureType.png, new byte[] { 161, 196, 248, 255 }),
    new("Kyron Saif", File.ReadAllBytes("/home/jacob/Downloads/k.png"), PictureType.png, new byte[] { 114, 213, 174, 255 }),
    new("Lucky Black", File.ReadAllBytes("/home/jacob/Downloads/b.png"), PictureType.png, new byte[] { 161, 196, 248, 255 }),
    new("quadro", Array.Empty<byte>(), PictureType.none, new byte[] { 229, 177, 233, 255  }),
    new("DarkPikachu", File.ReadAllBytes("/home/jacob/Downloads/tc.png"), PictureType.png, new byte[] { 181, 132, 95, 255 }),
    new("etile", Array.Empty<byte>(), PictureType.none, new byte[] { 181, 132, 95, 255 }),
    new("tranquillity", Array.Empty<byte>(), PictureType.none, new byte[] { 161, 196, 248, 255 }),
    new("Troler", File.ReadAllBytes("/home/jacob/Downloads/tr.png"), PictureType.png, new byte[] { 161, 196, 248, 255 }),
    new("xnq", File.ReadAllBytes("/home/jacob/Downloads/x.png"), PictureType.png, new byte[] { 161, 196, 248, 255 }),
    new("JacobTech Bot", Array.Empty<byte>(), PictureType.none, new byte[] { 114, 213, 174, 255 })
};
*/
//Matrix.ZipJsonChatExport(File.OpenRead("/home/jacob/Downloads/matrix - Luski Project Discussion - Chat Export - 2024-04-13T18-14-08.180Z.zip"), 2, 0, AMap, BMap, BMapL, new(), EncoderType.UTF16, 0);


if (!Tables.Roles.TryRead(Roles.ID, out _, Roles.ID.CreateParameter(0)))
{
    Tables.Roles.Insert(
        Roles.ID.CreateParameter(0),
        Roles.DisplayName.CreateParameter("Members"),
        Roles.Description.CreateParameter("The default role for the server. Everybody will have this role."),
        Roles.ServerPermissions.CreateParameter(
            ServerPermission.ViewChannels |
            ServerPermission.ViewCategories |
            ServerPermission.SendMessages |
            ServerPermission.SendFiles |
            ServerPermission.ChannelPings |
            ServerPermission.ServerPings |
            ServerPermission.PingSomeone |
            ServerPermission.ReadMessageHistory |
            ServerPermission.UseServerCommands |
            ServerPermission.JoinVoice |
            ServerPermission.SpeakInVoice |
            ServerPermission.ViewThis)
    );
}

long[] keys = Tables.Sessions.ReadColumn(Sessions.SessionKey);
foreach (long key in keys)
{
    Tables.Keys.DeleteRow(Keys.ID.CreateParameter(key));
}



Tables.Sessions.DeleteRows();
Tables.Users.CreateCommand().WithValue(Users.Status, UserStatus.Offline).Update();


foreach (AltServerRow Server in Tables.AltServers.ReadRows())
{
    //TODO add code to open a live WSS connection with all other servers on the network
}



WSS.Init(LuskiFunctions.Config.IPv4WSS, LuskiFunctions.Config.IPv4PortWSS, LuskiFunctions.Config.IPv4_URL_WSS!, LuskiFunctions.Config.IPv4SecureWSS, LuskiFunctions.Config.IPv6WSS, LuskiFunctions.Config.IPv6_URL_WSS, LuskiFunctions.Config.IPv6PortWSS, LuskiFunctions.Config.IPv6SecureWSS);

WebApplicationBuilder builder = WebApplication.CreateBuilder( args );

// Add services to the container.

builder.Services.AddControllers();
builder.Services.AddProblemDetails();
builder.Services.AddApiVersioning(
                    options =>
                    {
                        // reporting api versions will return the headers
                        // "api-supported-versions" and "api-deprecated-versions"
                        options.ReportApiVersions = true;
                        
                        /*    Sunset example
                        options.Policies.Sunset( 1 )
                                        //.Effective( DateTimeOffset.Now.Subtract( new TimeSpan(9999999) ) )
                                        .Effective( DateTimeOffset.Now.AddDays( 2 ) )
                                        .Link( "policy.html" )
                                            .Title( "Versioning Policy" )
                                            .Type( "text/html" );
                                            */
                    } )
                .AddMvc()
                .AddApiExplorer(
                    options =>
                    {
                        // add the versioned api explorer, which also adds IApiVersionDescriptionProvider service
                        // note: the specified format code will format the version as "'v'major[.minor][-status]"
                        options.GroupNameFormat = "'v'VVV";

                        // note: this option is only necessary when versioning by url segment. the SubstitutionFormat
                        // can also be used to control the format of the API version in route templates
                        options.SubstituteApiVersionInUrl = true;
                    } );
builder.Services.Configure<ApiBehaviorOptions>(options =>
{
    options.InvalidModelStateResponseFactory = actionContext => 
    {
        string content;
        using (StreamReader reader = new(actionContext.HttpContext.Request.Body))
        {
            content = reader.ReadToEndAsync().Result;
        }

        if (string.IsNullOrEmpty(content)) return new ObjectResult(new STC()
        {
            Error = ErrorCode.MissingPostData,
            ErrorMessage = "The post information was missing"
        })
        {
            StatusCode = 403
        };
        return new ObjectResult(new STC()
        {
            Error = ErrorCode.InvalidPostData,
            ErrorMessage = "The provided post information was invalid"
        })
        {
            StatusCode = 403
        };
    };
});
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddTransient<IConfigureOptions<SwaggerGenOptions>, ConfigureSwaggerOptions>();
builder.Services.AddSwaggerGen(
    options =>
    {
        // add a custom operation filter which sets default values
        options.OperationFilter<SwaggerDefaultValues>();

        string fileName = typeof(Program).Assembly.GetName().Name + ".xml";
        string filePath = Path.Combine(AppContext.BaseDirectory, fileName);

        // integrate xml comments
        options.IncludeXmlComments(filePath, true);
    } );

WebApplication app = builder.Build();
// Configure the HTTP request pipeline.

app.UseSwagger();
//app.UseHttpLogging();
app.UseSwaggerUI(
    options =>
    {
        IReadOnlyList<ApiVersionDescription> descriptions = app.DescribeApiVersions();

        // build a swagger endpoint for each discovered API version
        foreach ( ApiVersionDescription description in descriptions )
        {
            string url = $"/swagger/{description.GroupName}/swagger.json";
            string name = description.GroupName.ToUpperInvariant();
            options.SwaggerEndpoint(url, name);
        }
        options.InjectStylesheet("/www/css/SwaggerDark.css");
        options.DocumentTitle = "Luski Server API";
    } );
app.MapGet("/www/css/SwaggerDark.css", async (CancellationToken t) =>
{
    byte[] css = await File.ReadAllBytesAsync("www/css/SwaggerDark.css", t);
    return Results.File(css, "text/css");
}).ExcludeFromDescription();
app.UseHttpsRedirection();
app.UseAuthorization();
app.MapControllers();
app.Run();